'****************************************************************
'*  Name    : HRSOut_K40.inc                                    *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 03/02/2017                                        *
'*  Version : 1.0                                               *
'*  Purpose : HRSOut remplacement for 5 UARTs for PICxxK40      *
'*  Notes   : For PDS version 3.5.9.0                           *
'****************************************************************
'
' Version for the PIC18FxxK40 series
'  Date    : 20/03/2017
'  Version : 1.1
' Adapted for K20 series. When the PIC has a single UART.
'
'-------------------------------------------------------------------------------
'    $define _DisableAllinterrupt_
'    $define _Disable2StopBits_
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es un software libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' trs aprci.
'-------------------------------------------------------------------------------
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
' PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHOR OR COPYRIGHT
' HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
' OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'###############################################################################

$ifndef _HRSOutTwoStopBitsInc_
    $define _HRSOutTwoStopBitsInc_    
'===============================================================================
    $ifndef _DisableAllinterrupt_
    $define _EnableUARTinterrupt_
    $endif
'===============================================================================
    $ifndef _Disable2StopBits_
    $define _2StopBits_
    $endif
'*******************************************************************************
    #ifdef __SYSCOM_HRSOUT1_REQ_
'===============================================================================
    #Disable HRSOut             ' Bypass the compiler's HRSOut library subroutine
'===============================================================================
    GoTo OverHRSOut1Exit
'-------------------------------------------------------------------------------
Asm
HRSOut
EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif 

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        PRODH = INTCON
        INTCONBits_GIE = 0              ' Clear the GIE bit.
        #endif
    $endif
    
    ' This code take care of the different Banks of TXxSTA.
    ' Waiting UART ready.
    While 1 = 1
        $ifdef _TX1STA
        PRODL = TX1STA
        $else
        PRODL = TXSTA    
        $endif
        If PRODL.1 = 1 Then Break
    Wend
    
    ' To avoid some old PIC's errata the bits are set after sent the last byte.
    $ifdef _2StopBits_
    $ifdef _TX1STA
    TX1STABits_TX9 = 1                  ' Nine bit mode for 2 stop bits for transmision.
    TX1STABits_TX9D = 1                 ' Send the first stop bit. The second stop bit is set by the UART.
    $else
    TXSTABits_TX9 = 1
    TXSTABits_TX9D = 1
    $endif
    $endif

    $ifdef _TX1REG
    TX1REG = WREG                       ' Start sending the Byte
    $else
    TXREG = WREG                        ' Start sending the Byte
    $endif

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        If PRODH.7 = 1 Then
            INTCONBits_GIE = 1          ' Restore the GIE interrupt condition.
        EndIf
        #endif
    $endif

    Return 
'-------------------------------------------------------------------------------
    #endif  ' __SYSCOM_HRSOUT1_REQ_
'===============================================================================
$define CleanUART1()      '
    $ifdef _RC1STA        '
    Clear RC1STABits_CREN ' 
    Set RC1STABits_CREN   ' 
    WREG = RC1REG         '
    $else                 '
    Clear RCSTABits_CREN  ' 
    Set RCSTABits_CREN    ' 
    WREG = RCREG          '
    $endif

'-------------------------------------------------------------------------------
OverHRSOut1Exit:
'*******************************************************************************
'*******************************************************************************
$if _uart > 1
    #ifdef __SYSCOM_HRSOUT2_REQ_
'===============================================================================
    #Disable HRSOut2             ' Bypass the compiler's HRSOut2 library subroutine
'===============================================================================
    GoTo OverHRSOut2Exit
'-------------------------------------------------------------------------------
Asm
HRSOUT@2
EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif 

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        PRODH = INTCON
        INTCONBits_GIE = 0              ' Clear the GIE bit.
        #endif
    $endif

    ' This code take care of the different Banks of TXxSTA.
    ' Waiting UART ready.
    While 1 = 1
        PRODL = TX2STA
        If PRODL.1 = 1 Then Break
    Wend

    ' To avoid some old PIC's errata the bits are set after sent the last byte.
    $ifdef _2StopBits_
    TX2STABits_TX9 = 1                  ' Nine bit mode for 2 stop bits for transmision.
    TX2STABits_TX9D = 1                 ' Send the first stop bit. The second stop bit is set by the UART.
    $endif

    TX2REG = WREG                      ' Start sending the Byte

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        If PRODH.7 = 1 Then
            INTCONBits_GIE = 1          ' Restore the GIE interrupt condition.
        EndIf
        #endif
    $endif

    Return 
'-------------------------------------------------------------------------------
    #endif  ' __SYSCOM_HRSOUT2_REQ_
'===============================================================================
$define CleanUART2()      '
    Clear RC2STABits_CREN ' 
    Set RC2STABits_CREN   ' 
    WREG = RC2REG

'-------------------------------------------------------------------------------
OverHRSOut2Exit:
$endif
'*******************************************************************************
'*******************************************************************************
$if _uart > 2
    #ifdef __SYSCOM_HRSOUT3_REQ_
'===============================================================================
    #Disable HRSOut3             ' Bypass the compiler's HRSOut3 library subroutine
'===============================================================================
    GoTo OverHRSOut3Exit
'-------------------------------------------------------------------------------
Asm
HRSOUT@3
EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif 

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        PRODH = INTCON
        INTCONBits_GIE = 0                  ' Clear the GIE bit.
        #endif
    $endif

    ' This code take care of the different Banks of TXxSTA.
    ' Waiting UART ready.
    While 1 = 1
        PRODL = TX3STA
        If PRODL.1 = 1 Then Break
    Wend

    ' To avoid some old PIC's errata the bits are set after sent the last byte.
    $ifdef _2StopBits_
    TX3STABits_TX9 = 1                  ' Nine bit mode for 2 stop bits for transmision.
    TX3STABits_TX9D = 1                 ' Send the first stop bit. The second stop bit is set by the UART.
    $endif

    TX3REG = WREG                       ' Start sending the Byt

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        If PRODH.7 = 1 Then
            INTCONBits_GIE = 1          ' Restore the GIE interrupt condition.
        EndIf
        #endif
    $endif

    Return 
'-------------------------------------------------------------------------------
    #endif  ' __SYSCOM_HRSOUT3_REQ_
'===============================================================================
$define CleanUART3()      '
    Clear RC3STABits_CREN ' 
    Set RC3STABits_CREN   ' 
    WREG = RC3REG

'-------------------------------------------------------------------------------
OverHRSOut3Exit:
$endif
'*******************************************************************************
'*******************************************************************************
$if _uart > 3
    #ifdef __SYSCOM_HRSOUT4_REQ_
'===============================================================================
    #Disable HRSOut4             ' Bypass the compiler's HRSOut4 library subroutine
'===============================================================================
    GoTo OverHRSOut4Exit
'-------------------------------------------------------------------------------
Asm
HRSOUT@4
EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif 

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        PRODH = INTCON
        INTCONBits_GIE = 0                  ' Clear the GIE bit.
        #endif
    $endif

    ' This code take care of the different Banks of TXxSTA.
    ' Waiting UART ready.
    While 1 = 1
        PRODL = TX4STA
        If PRODL.1 = 1 Then Break
    Wend

    ' To avoid some old PIC's errata the bits are set after sent the last byte.
    $ifdef _2StopBits_
    TX4STABits_TX9 = 1                  ' Nine bit mode for 2 stop bits for transmision.
    TX4STABits_TX9D = 1                 ' Send the first stop bit. The second stop bit is set by the UART.
    $endif

    TX4REG = WREG                       ' Start sending the Byt

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        If PRODH.7 = 1 Then
            INTCONBits_GIE = 1              ' Restore the GIE interrupt condition.
        EndIf
        #endif
    $endif

    Return 
'-------------------------------------------------------------------------------
    #endif  ' __SYSCOM_HRSOUT4_REQ_
'===============================================================================
$define CleanUART4()      '
    Clear RC4STABits_CREN ' 
    Set RC4STABits_CREN   ' 
    WREG = RC4REG

'-------------------------------------------------------------------------------
OverHRSOut4Exit:
$endif
'*******************************************************************************
'*******************************************************************************
$if _uart > 4
    #ifdef __SYSCOM_HRSOUT5_REQ_
'===============================================================================
    #Disable HRSOut5             ' Bypass the compiler's HRSOut5 library subroutine
'===============================================================================
    GoTo OverHRSOut5Exit
'-------------------------------------------------------------------------------
Asm
HRSOUT@5
EndAsm
    #ifdef watchdog_req
    Clrwdt
    #endif 

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        PRODH = INTCON
        INTCONBits_GIE = 0              ' Clear the GIE bit.
        #endif
    $endif

    ' This code take care of the different Banks of TXxSTA.
    ' Waiting UART ready.
    While 1 = 1
        PRODL = TX5STA
        If PRODL.1 = 1 Then Break
    Wend

    ' To avoid some old PIC's errata the bits are set after sent the last byte.
    $ifdef _2StopBits_
    TX5STABits_TX9 = 1                  ' Nine bit mode for 2 stop bits for transmision.
    TX5STABits_TX9D = 1                 ' Send the first stop bit. The second stop bit is set by the UART.
    $endif

    TX5REG = WREG                       ' Start sending the Byt

    $ifdef _EnableUARTinterrupt_
        #ifdef __INTERRUPTS_ENABLED
        If PRODH.7 = 1 Then
            INTCONBits_GIE = 1          ' Restore the GIE interrupt condition.
        EndIf
        #endif
    $endif

    Return 
'-------------------------------------------------------------------------------
    #endif  ' __SYSCOM_HRSOUT5_REQ_
'===============================================================================
$define CleanUART5()      '
    Clear RC5STABits_CREN ' 
    Set RC5STABits_CREN   ' 
    WREG = RC5REG

'-------------------------------------------------------------------------------
OverHRSOut5Exit:
$endif
'===============================================================================
'===============================================================================
' Clear the UART before to use.
    #ifdef __SYSCOM_HRSOUT1_REQ_
    CleanUART1()
    #endif

    $if _uart > 1
    #ifdef __SYSCOM_HRSOUT2_REQ_
    CleanUART2()
    #endif
    $endif    

    $if _uart > 2
    #ifdef __SYSCOM_HRSOUT3_REQ_
    CleanUART3()
    #endif    
    $endif

    $if _uart > 3
    #ifdef __SYSCOM_HRSOUT4_REQ_
    CleanUART4()
    #endif
    $endif

    $if _uart > 4
    #ifdef __SYSCOM_HRSOUT5_REQ_
    CleanUART5()
    #endif
    $endif
'-------------------------------------------------------------------------------
$endif  ' _HRSOutTwoStopBitsInc_
'-------------------------------------------------------------------------------    
    
    
     
