'****************************************************************
'*  Name    : LCD_ST7036.inc                                    *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2013 [AFreixanet 2013]              *
'*          : All Rights Reserved                               *
'*  Date    : 17/01/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Driver for parallel Display with ST7036 interface *
'*          : Liquid Crystal controller ST7036                  *
'*          : Writen for the AMI18 LCD Shield  (GEVO)           *
'* Compiler : Powered by the Proton Development Suite V3575     *
'*          : Based on the Les's LCD code structure             *
'****************************************************************
'
' 20/01/2016 V1.1
' Remove 2 unused variables word.
' 26/03/2017 V1.2
' Change the address in eeprom to save the contrast datas at the end of eeprom memory.
'    Symbol EEADR_ContrastCode = _eeprom - 1
'    Symbol EEADR_ContrastData = _eeprom - 2
' 
'-------------------------------------------------------------------------------
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera utile,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
'------------------------------------------------------------------------------- 
$ifndef _ST7036_INC_
$define _ST7036_INC_
'===============================================================================

$define _LCD_ST7036_3V_

'===============================================================================
' Setup the LCD. These must be included in the program (8bits bus of for 4 bits bus)
(*
 For 8 bits Bus.
    Declare LCD_DTPin = PORTB.0
    Declare LCD_RSPin = PORTA.1     ' Could be another pin.
    Declare LCD_ENPin = PORTA.0     ' Could be another pin.
    Declare LCD_Interface = 8
    Declare LCD_Lines = 2
    Declare LCD_Type = Alphanumeric
    Declare LCD_CommandUs = 2000
    Declare LCD_DataUs = 50
'    $define LCD_Reverse_Bits 

 For 4 bits Bus. Connected in the low bibble.
    Declare LCD_DTPin = PORTB.0
    Declare LCD_RSPin = PORTA.1     ' Could be another pin.
    Declare LCD_ENPin = PORTA.0     ' Could be another pin.
    Declare LCD_Interface = 4
    Declare LCD_Lines = 2
    Declare LCD_Type = Alphanumeric
    Declare LCD_CommandUs = 2000
    Declare LCD_DataUs = 50
'    $define LCD_Reverse_Bits  

 For 4 bits Bus. Connected in the High bibble.
    Declare LCD_DTPin = PORTB.4
    Declare LCD_RSPin = PORTA.1     ' Could be another pin.
    Declare LCD_ENPin = PORTA.0     ' Could be another pin.
    Declare LCD_Interface = 4
    Declare LCD_Lines = 2
    Declare LCD_Type = Alphanumeric
    Declare LCD_CommandUs = 2000
    Declare LCD_DataUs = 50
'    $define LCD_Reverse_Bits
*)
'===============================================================================
' Replacement routine for the compiler's Print command for an COG alphanumeric LCD
' Using an 8-bit and 4-bit interface.
' written by A. Freixanet 15/01/2016
'   
    #disable Print                              ' Disable the compiler's print library routine

'-------------------------------------------------------------------------------
    GoTo OverST7036_Lib                         '  Jump over the subroutine
'-------------------------------------------------------------------------------
' Create system variables for the routines
'    
'    Dim BPF  As Byte System
'    Dim PP0  As Byte System
'    Dim PP0H As Byte System
'    Dim PP3  As Byte System
'    Dim PP3H As Byte System
'
    Dim _mContrast      As Byte System
    Dim _mLCD_Index     As Byte System
    Dim _DisplayControl As Byte System

' Create some aliases to make the code's operation clearer
'
    Dim Hitachi8_bByteOut     As PP3
    Dim LCD_WREGStore         As PP3H
    Dim Hitachi8_tInitialised As BPF.1
    Dim Hitachi8_tComOrData   As BPF.0

    Dim Hitachi4_bByteOut     As PP3
    Dim Hitachi4_tInitialised As BPF.1
    Dim Hitachi4_tComOrData   As BPF.0

' Local Bytes variables specific for this library-
'
' Definition of local Bytes
$ifndef _LVBS_Block1_
    $define _LVBS_Block1_
    Dim LVSByte1 As Byte System     ' Local Variable System Byte1
    Dim LVSByte2 As Byte System     ' Local Variable System Byte2
    Dim LVSByte3 As Byte System     ' Local Variable System Byte3
    Dim LVSByte4 As Byte System     ' Local Variable System Byte4
$endif
'$ifndef _LVBS_Block7036_
'    $define _LVBS_Block7036_
'    Dim LVSByte1ACZ As Byte System     ' Local Variable System Byte1
'    Dim LVSByte2ACZ As Byte System     ' Local Variable System Byte2
'    Dim LVSByte3ACZ As Byte System     ' Local Variable System Byte3
'    Dim LVSByte4ACZ As Byte System     ' Local Variable System Byte4
'$endif

'' Local Words variables
'$ifndef _LVWS_Block1_
'    $define _LVWS_Block1_
'    Dim LVSWord1 As Word System     ' Local Variable System Word1
'    Dim LVSWord2 As Word System     ' Local Variable System Word2
'$endif                   

'-------------------------------------------------------------------------------
'===============================================================================
'-------------------------------------------------------------------------------
' Function Name: Command/flag definitions
'
$define csLCD_ClearDisplay			    $01   ' Command And Setting ST7036
$define csLCD_ReturnHome				$02   ' Command And Setting ST7036
'
' flags for display entry mode
$define csLCD_EntryModeSetNOShift		    $04   'Command And Setting ST7036
$define csLCD_EntryMSShiftDisplayToLeft     $07   'Command and Setting ST7036 
$define csLCD_EntryMSShiftDisplayToRight    $05   'Command And Setting ST7036
'                      
' flags for display on/off control
$define cLCD_DisplayControl	  $08         ' Command

$define csLCD_ControlDisp_ON  %00001100   ' $0C, Command + function, NO cursor, NO Blink
'$define csLCD_ControlDisp_OFF %00001000   ' $08, Command + function, NO Cursor, NO Blink

$define sLCD_Display_ON      $04  ' %00000100, Setting, use | function
$define sLCD_Display_OFF     $FB  ' %11111011, Setting, use & function 
$define sLCD_Cursor_ON       $02  ' %00000010, Setting, use | function
$define sLCD_Cursor_OFF      $FD  ' %11111101, Setting, use & function   
$define sLCD_Blink_ON        $01  ' %00000001, Setting, use | function
$define sLCD_Blink_OFF       $FE  ' %11111110, Setting, use & function
$define sLCD_DisplayCOntrol_AllCursorOFF  $04  ' Setting 
'
' flags for Functions
$define csLCD_FunctionSet_2Line_Basic       $38     ' Command and Settings 
$define csLCD_FunctionSet_2Line_Extended    $39     ' Command And Settings
$define csLCD_FunctionSet_1Line_Basic       $34     ' Command and Settings 
$define csLCD_FunctionSet_1Line_Extended    $35     ' Command And Settings
'
$ifdef _LCD_ST7036_3V_
$define csLCD_FunctionSet_Table0    %00101000   'Command and Settings ST7036
$define csLCD_FunctionSet_Table1    %00101001   'Command and Settings ST7036
$define csLCD_FunctionSet_Table2    %00101010   'Command and Settings ST7036
$define csLCD_FunctionSet1LineDH    %00100100   'Command and Settings ST7036
$else
    $error "Define the FunctionSet for the new LCD"
$endif
'
$define cLCD_SetDDRAMADDR		$80  ' %10000000, mCommand  (ST7032)(ST7036)
$define csLCD_SetDDRAMADDR1L    $80  ' %10000000, Command And Settings	
$define csLCD_SetDDRAMADDR2L    $C0  ' %11000000, Command And Settings
'
$define cLCD_DisplayCursorSHIFT	$10  ' Command ST7036

$define csLCD_CursorToLeft      $10  ' Command +  Setting ST7036
$define csLCD_CursorToRight     $14  ' Command +  Setting ST7036
$define csLCD_DisplayToLeft     $18  ' Command +  Setting ST7036
$define csLCD_DisplayToRight    $1C  ' Command +  Setting ST7036

$define sLCD_CursorToLeft       $00  ' Setting ST7036
$define sLCD_CursorToRight      $04  ' Setting ST7036
$define sLCD_DisplayToLeft      $08  ' Setting ST7036
$define sLCD_DisplayToRight     $0C  ' Setting ST7036
'
' Address of the patterns in CGRAM                                   
$define cLCD_SetCGRAMADDR       $40  ' Command 
$define csLCD_CGRAM_ADDR0       $40  ' %01000000, Command And Settings ST7036
$define CGRAM0                  $40  ' Command + Address Pattern 1
$define CGRAM1                  $48  ' Command + Address Pattern 2
$define CGRAM2                  $50  ' Command + Address Pattern 3
$define CGRAM3                  $58  ' Command + Address Pattern 4
$define CGRAM4                  $60  ' Command + Address Pattern 5
$define CGRAM5                  $68  ' Command + Address Pattern 6
$define CGRAM6                  $70  ' Command + Address Pattern 7
$define CGRAM7                  $78  ' Command + Address Pattern 8
'
$define cLCD_Bias_OSC_Control   $10  'Command ST7032
$define cLCD_Bias_Control       $14  'Command ST7036 
$define sLCD_Bias1_4 		    $08	 'Setting ST7036			 
$define sLCD_Bias1_5 			$00  'Setting ST7036
$define sLCD_FX_1_2Lines 		$00  'Setting, use & ST7036
$define sLCD_FX_3Lines 			$01  'Setting ST7036
'					
' Internal frequency adjust for VDD = 3.3 V
$ifdef _LCD_ST7036_3V_
$define sLCD_OSC_122	$00	 ' Setting (ST7032)			
$define sLCD_OSC_131	$01	 ' Setting (ST7032)			
$define sLCD_OSC_144	$02	 ' Setting (ST7032)			
$define sLCD_OSC_161	$03	 ' Setting (ST7032)			
$define sLCD_OSC_183	$04	 ' Setting (ST7032)		
$define sLCD_OSC_221	$05	 ' Setting (ST7032)			
$define sLCD_OSC_274	$06	 ' Setting (ST7032)			
$define sLCD_OSC_347	$07	 ' Setting (ST7032)
$endif
'
' Internal frequency adjust for VDD = 5.0 V
$ifdef _LCD_AMI18_5V_
$define sLCD_OSC_120	$00  ' Setting (ST7032)			
$define sLCD_OSC_133	$01  ' Setting (ST7032)			
$define sLCD_OSC_149	$02  ' Setting (ST7032)			
$define sLCD_OSC_167	$03  ' Setting (ST7032)			
$define sLCD_OSC_192	$04  ' Setting (ST7032)* Default			
$define sLCD_OSC_227	$05  ' Setting (ST7032)			
$define sLCD_OSC_277	$06  ' Setting (ST7032)			
$define sLCD_OSC_347	$07  ' Setting (ST7032)
$endif
'
$define cICON_RAMAddressSet		$40  ' Command (ST7032)(ST7036)		
'
$define cLCD_ICON_Contrast_HIGH_Byte  $50   ' Command ST7036
$define cLCD_PowerICON_Control        $50   ' Command ST7036
$define sLCD_ICON_ON				  $08   ' Setting, use | function   (ST7036)
$define sLCD_ICON_OFF				  $F7   ' Setting, use & function   (ST7036)
$define sLCD_Booster_ON				  $04   ' Setting, use | function * (ST7036)
$define sLCD_Booster_OFF			  $FB   ' Setting, use & function   (ST7036)
$define sLCD_Contrast_HIGH_Byte_AMI18 $01   ' Setting, use | function   (ST7036)
$define sLCD_Contrast_HIGH_Byte_MASK  $03   ' Only used for bit masking (ST7036)
'
$define cLCD_Follower_Control		 $60   ' Command (ST7032) ST7036
$define sLCD_Follower_ON			 $08   ' Setting (ST7032)*  ST7036			
$define sLCD_Follower_OFF			 $F7   ' Setting (ST7032)   ST7036, use &			
$define sLCD_Rab_1_00				 $00   ' Setting (ST7032)   ST7036			
$define sLCD_Rab_1_25				 $01   ' Setting (ST7032)   ST7036			
$define sLCD_Rab_1_50				 $02   ' Setting (ST7032)   ST7036			
$define sLCD_Rab_1_80				 $03   ' Setting (ST7032)   ST7036			
$define sLCD_Rab_2_00				 $04   ' Setting (ST7032)*  ST7036			
$define sLCD_Rab_2_50				 $05   ' Setting (ST7032)   ST7036			
$define sLCD_Rab_3_00				 $06   ' Setting (ST7032)   ST7036			
$define sLCD_Rab_3_75				 $07   ' Setting (ST7032)   ST7036			
'
$define cLCD_Contrast_LOW_Byte	     $70   ' Command (ST7032)
$define cLCD_ContrastLowByte_Control $70   ' Command (ST7036)
$define sLCD_Contrast_LOW_Byte_MASK  $0F   ' Only used for bit masking (ST7036)
'
'-------------------------------------------------------------------------------
' Definition of the FUNCTION SET for 4 Bits BUS
' 2 Lines 5x7 dot
Symbol _FUNCTION_SET_4B_2L_T0 = $28    ' 4 bits 2 lines, For Table 0
Symbol _FUNCTION_SET_4B_2L_T1 = $29    ' 4 bits 2 lines, For Table 1   
Symbol _FUNCTION_SET_4B_2L_T2 = $2A    ' 4 bits 2 lines, For Table 2  

' 1 Line 5x16 dot
Symbol _FUNCTION_SET_4B_1L_T0 = $24    ' 4 bits 1 lines, For Table 0   
Symbol _FUNCTION_SET_4B_1L_T1 = $25    ' 4 bits 1 lines, For Table 1   
Symbol _FUNCTION_SET_4B_1L_T2 = $26    ' 4 bits 1 lines, For Table 2 

' Definition of the FUNCTION SET for 8 Bits BUS
' 2 Lines 5x7 dot
Symbol _FUNCTION_SET_8B_2L_T0 = 0x38   ' 4 bits 2 lines, For Table 0
Symbol _FUNCTION_SET_8B_2L_T1 = 0x39   ' 4 bits 2 lines, For Table 1
Symbol _FUNCTION_SET_8B_2L_T2 = 0x3A   ' 4 bits 2 lines, For Table 2

' 1 Line 5x16 dot
Symbol _FUNCTION_SET_8B_1L_T0 = 0x34   ' 4 bits 1 line, For Table 0
Symbol _FUNCTION_SET_8B_1L_T1 = 0x35   ' 4 bits 1 line, For Table 1
Symbol _FUNCTION_SET_8B_1L_T2 = 0x36   ' 4 bits 1 line, For Table 2

'-------------------------------------------------------------------------------
' Function Name: Print Commands of the Proton Basic Compiler / Amicus18
' Syntax       : Print LcdCmd,LcdHome  (Command of Proton Basic)
' Notes        :
'
' Standard Commands
    $define LcdCmd $FE          ' Command mode byte
' Now Setting Byte, NO Page
    $define LcdClr $01          ' Clear Display 
    $define LcdHome $02         ' Return home (beginning of first line)
    $define LcdDisplayOff $08   ' Turn visible LCD off
    $define LcdDisplayOn  $0C   ' Turn visible LCD on
'    $define LcdUnderlineOff $0C ' Underline Cursor Off
'    $define LcdUnderlineOn $0E  ' Underline Cursor On
'    $define LcdBlinkingOff $0C  ' Blinking Box Cursor Off
'    $define LcdBlinkingOn $0F   ' Blinking Box Cursor On

'Set Table 0
    $define LcdCurMoveLeft $10  ' Move Cursor Left one position
    $define LcdCurMoveRight $14 ' Move Cursor Right one position
    $define LcdScrollRight $1C  ' Scroll entire screen Right one space
    $define LcdScrollLeft $18   ' Scroll entire screen Left one space

' Special Command
    $define LcdHome2 $C0        ' %11000000, Move cursor to beginning of second line

' Address of configuration in eeprom memory. (this addresses could be changed by the user).
    Symbol EEADR_ContrastCode = _eeprom - 1
    Symbol EEADR_ContrastData = _eeprom - 2    
'======= End of command/flag defenitions =======================================
'===============================================================================  
' Macros to Clear and Restore GIE
' Input     : 
' Output    : 
' Notes     : Use the new intuitive GIE bit clear and restore, if necessary.
'
' Create a system variable for the GIE routines
    $ifndef _xINTCON_Save_Var_
    $define _xINTCON_Save_Var_
    Dim xINTCON_Save As Byte System
    $endif

' This new command save and clear the GIE global interrupt.
' It is compiled only if the interrupts are enabled.
$ifndef GIE_Save
    $define GIE_Save()          '              
    #ifdef __INTERRUPTS_ENABLED '
    xINTCON_Save = INTCON       '
    INTCONbits_GIE = 0          '
    #endif

$endif

'-------------------------------------------------------------------------------    
' This new command restore the GIE global interrupt.
' It is compiled only if the interrupts are enabled.
$ifndef GIE_Restore
    $define GIE_Restore()       '
    #ifdef __INTERRUPTS_ENABLED '
    If xINTCON_Save.7 = 1 Then  '
        INTCONbits_GIE = 1      '
    EndIf                       '
    #endif

$endif

'-------------------------------------------------------------------------------
' Reset the LCD to force an initialization.
    $define LCD_Reset() ' 
    BPF = 0             '
    Cls
    
'*******************************************************************************   
'===============================================================================
' Print replacement routine.
Asm-
Print
Endasm-
    LCD_WREGStore = WREG                   ' Save the input byte. 

    #ifdef WatchDog_Req
    Clrwdt
    #endif
'===============================================================================
'#if (__LCD_INTERFACE == 8)
'' Start LCD HITACHI BUS 8 Bits
    ' Set the __LCD_ENPORT.__LCD_ENPIN to digital.
    #if (__LCD_ENPORT == PORTA)
        #if (__LCD_ENPIN == 0)
            $ifdef _ANSEL
            ANSEL.0 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.0 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 1)
            $ifdef _ANSEL
            ANSEL.1 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.1 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 2)
            $ifdef _ANSEL
            ANSEL.2 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.2 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 3)
            $ifdef _ANSEL
            ANSEL.3 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.3 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 5)
            $ifdef _ANSEL
            ANSEL.4 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.5 = 0
            $endif
        #endif
    #endif

    #if (__LCD_ENPORT == PORTB)
        #if (__LCD_ENPIN == 0)
            $ifdef _ANSELH
            ANSELH.4 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.0 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 1)
            $ifdef _ANSELH
            ANSELH.2 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.1 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 2)
            $ifdef _ANSELH
            ANSELH.0 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.2 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 3)
            $ifdef _ANSELH
            ANSELH.1 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.3 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 4)
            $ifdef _ANSELH
            ANSELH.3 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.5 = 0
            $endif
        #endif
    #endif

    #if (__LCD_ENPORT == PORTC)
        #if (__LCD_ENPIN == 2)
            $ifdef _ANSELC
            ANSELC.2 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 3)
            $ifdef _ANSELC
            ANSELC.3 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 4)
            $ifdef _ANSELC
            ANSELC.4 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 5)
            $ifdef _ANSELC
            ANSELC.5 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 6)
            $ifdef _ANSELC
            ANSELC.6 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 7)
            $ifdef _ANSELC
            ANSELC.7 = 0
            $endif
        #endif
    #endif

    $ifdef _PORTD
    #if (__LCD_ENPORT == PORTD)
        #if (__LCD_ENPIN == 0)
            $ifdef _ANSELD
            ANSELD.0 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 1)
            $ifdef _ANSELD
            ANSELD.1 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 2)
            $ifdef _ANSELD
            ANSELD.2 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 3)
            $ifdef _ANSELD
            ANSELD.3 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 4)
            $ifdef _ANSELD
            ANSELD.4 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 5)
            $ifdef _ANSELD
            ANSELD.5 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 6)
            $ifdef _ANSELD
            ANSELD.6 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 7)
            $ifdef _ANSELD
            ANSELD.7 = 0
            $endif
        #endif
    #endif
    $endif
    
    $ifdef _PORTE
    #if (__LCD_ENPORT == PORTE)
        #if (__LCD_ENPIN == 0)
            $ifdef _ANSEL
            ANSEL.5 = 0
            $endif
            $ifdef _ANSELE
            ANSELE.0 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 1)
            $ifdef _ANSEL
            ANSEL.6 = 0
            $endif
            $ifdef _ANSELE
            ANSELE.1 = 0
            $endif
        #endif
        #if (__LCD_ENPIN == 2)
            $ifdef _ANSEL
            ANSEL.7 = 0
            $endif
            $ifdef _ANSELE
            ANSELE.2 = 0
            $endif
        #endif
    #endif
    $endif

    Low __LCD_ENPORT.__LCD_ENPIN                ' Pull the EN pin to output low
'-------------------------------------------------------------------------------
    ' Set the __LCD_RSPORT.__LCD_RSPIN to digital.
    #if (__LCD_RSPORT == PORTA)
        #if (__LCD_RSPIN == 0)
            $ifdef _ANSEL
            ANSEL.0 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.0 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 1)
            $ifdef _ANSEL
            ANSEL.1 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.1 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 2)
            $ifdef _ANSEL
            ANSEL.2 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.2 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 3)
            $ifdef _ANSEL
            ANSEL.3 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.3 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 5)
            $ifdef _ANSEL
            ANSEL.4 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.5 = 0
            $endif
        #endif
    #endif

    #if (__LCD_RSPORT == PORTB)
        #if (__LCD_RSPIN == 0)
            $ifdef _ANSELH
            ANSELH.4 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.0 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 1)
            $ifdef _ANSELH
            ANSELH.2 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.1 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 2)
            $ifdef _ANSELH
            ANSELH.0 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.2 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 3)
            $ifdef _ANSELH
            ANSELH.1 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.3 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 4)
            $ifdef _ANSELH
            ANSELH.3 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.5 = 0
            $endif
        #endif
    #endif

    #if (__LCD_RSPORT == PORTC)
        #if (__LCD_RSPIN == 2)
            $ifdef _ANSELC
            ANSELC.2 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 3)
            $ifdef _ANSELC
            ANSELC.3 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 4)
            $ifdef _ANSELC
            ANSELC.4 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 5)
            $ifdef _ANSELC
            ANSELC.5 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 6)
            $ifdef _ANSELC
            ANSELC.6 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 7)
            $ifdef _ANSELC
            ANSELC.7 = 0
            $endif
        #endif
    #endif

    $ifdef _PORTD
    #if (__LCD_RSPORT == PORTD)
        #if (__LCD_RSPIN == 0)
            $ifdef _ANSELD
            ANSELD.0 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 1)
            $ifdef _ANSELD
            ANSELD.1 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 2)
            $ifdef _ANSELD
            ANSELD.2 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 3)
            $ifdef _ANSELD
            ANSELD.3 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 4)
            $ifdef _ANSELD
            ANSELD.4 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 5)
            $ifdef _ANSELD
            ANSELD.5 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 6)
            $ifdef _ANSELD
            ANSELD.6 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 7)
            $ifdef _ANSELD
            ANSELD.7 = 0
            $endif
        #endif
    #endif
    $endif
    
    $ifdef _PORTE
    #if (__LCD_RSPORT == PORTE)
        #if (__LCD_RSPIN == 0)
            $ifdef _ANSEL
            ANSEL.5 = 0
            $endif
            $ifdef _ANSELE
            ANSELE.0 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 1)
            $ifdef _ANSEL
            ANSEL.6 = 0
            $endif
            $ifdef _ANSELE
            ANSELE.1 = 0
            $endif
        #endif
        #if (__LCD_RSPIN == 2)
            $ifdef _ANSEL
            ANSEL.7 = 0
            $endif
            $ifdef _ANSELE
            ANSELE.2 = 0
            $endif
        #endif
    #endif
    $endif

    Low __LCD_RSPORT.__LCD_RSPIN                ' Pull the RS pin to output Low
    '===========================================================================
#if (__LCD_INTERFACE == 8)
' Start LCD HITACHI BUS 8 Bits
    ' Set the LCD PORT to digital.
    #if (__LCD_DTPORT == PORTA)
        $ifdef _ANSEL
        ANSEL = %11100000
        $endif
        $ifdef _ANSELA
        ANSELA = 0
        $endif    
    #endif

    #if (__LCD_DTPORT == PORTB)
        $ifdef _ANSELH
        ANSELH = 0
        $endif
        $ifdef _ANSELB
        ANSELB = 0
        $endif    
    #endif

    $ifdef _PORTC
    #if (__LCD_DTPORT == PORTC)
        $ifdef _ANSELC
        ANSELC = 0
        $endif    
    #endif
    $endif    

    $ifdef _PORTD
    #if (__LCD_DTPORT == PORTD)
        $ifdef _ANSELD
        ANSELD = 0
        $endif    
    #endif
    $endif

    Output __LCD_DTPORT                         ' Set the port to output
    '---------------------------------------------------------------------------
    If Hitachi8_tInitialised = 0 Then           ' Has LCD been inititalised?
        DelayMS 150                             ' No. So wait 150ms
        Hitachi8_bByteOut = $38                 ' Start Initialise the LCD
        GoSub Hitachi8_SetData                  ' Send init
'        DelayUS 2000                            ' Wait 5ms
        DelayUS 5000                            ' Wait 5ms
        GoSub Hitachi8_SetData                  ' Send init again
        DelayUS 100                             ' Wait 100us

        WREG = _FUNCTION_SET_8B_2L_T1           ' Function_Set: 8-bit=$39, Table 1
        GoSub Hitachi8_SetCommand               ' 
        WREG = $14                              ' Bias set: BS=0, FX=0                    %0001 0100  
        GoSub Hitachi8_SetCommand               ' 
        WREG = $5E                              ' $55 Power control: Ion=0, Bon=1, C5=0, C4=1 %0101 0101
        GoSub Hitachi8_SetCommand               ' 
        WREG = $6A                              ' $6D Follower control: Fon=1, Rab2=1, Rab1=0, Rab0=1           %0110 1101
        GoSub Hitachi8_SetCommand               ' 
        '-----------------------------------------------------------------------
        'Setting the LCD Contrast: The contrast value could be written or not in the eeprom memory.
        ' After Reset, the code read the eeprom memory if there is a correct contrast value.
        _mLCD_Index = ERead EEADR_ContrastCode      ' Read the code value saved in the eeprom memory.
        If _mLCD_Index = $5A Then                   ' Check if there is a true contrast value?
            _mLCD_Index = ERead EEADR_ContrastData  ' Yes, there is..., load it.
            _mLCD_Index = _mLCD_Index & 15          ' No more than 15.
            _mContrast = _mLCD_Index                ' Save the value for current library contrast.
            WREG = _mLCD_Index | $70                ' Mix the command with value.
            GoSub Hitachi4_SetCommand               ' Now write the correct value to the LCD.
        Else                                        ' If not contrast written in the eeprom then...
            _mContrast = 7                          ' Load a default contrast.
            WREG = $77                              ' Mix the address...
            GoSub Hitachi4_SetCommand               ' Writing the value to the LCD.
            EWrite EEADR_ContrastData,[_mContrast]  ' It is the default value, write it to the eeprom.
            EWrite EEADR_ContrastCode,[$5A]         ' Write the control too.        
        EndIf
        '-----------------------------------------------------------------------
        #ifdef __LCD_LINES                      ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)               ' Yes. So...
                WREG = _FUNCTION_SET_8B_1L_T0   ' Set for 4-bit mode, 1 line, 5x16 font
            #else
                WREG = _FUNCTION_SET_8B_2L_T0   ' Set for 4-bit mode, 2+ lines, 5x7 font
            #endif
        #else                                   ' Otherwise... Default to 2 lines
            WREG = _FUNCTION_SET_8B_2L_T0       ' Set for 4-bit mode, 2+ lines, 5x7 font
        #endif
        GoSub Hitachi8_SetCommand               ' Write Function Set

        WREG = $0C                              ' Display Control: %0000 1100, encender el display
        _DisplayControl = WREG                  ' Valor de inicio: D=1, C=0, B=0, Display ON, Cursor OFF
        GoSub Hitachi8_SetCommand               ' 
        WREG = $01                              ' Clear display
        GoSub Hitachi8_SetCommand               ' 
        WREG = $06                              ' Entry Mode Set, CURSOR MOVES TO RIGHT
        GoSub Hitachi8_SetCommand               ' 
        WREG = $02                              ' Return Home. Set DDRAM address to "00H" from AC and return cursor to its original
        GoSub Hitachi4_SetCommand               ' position if shifted. The contents of DDRAM are not changed.
                                                ' Sometimes the ST7036 is sensitive with bad reset, then this command saves this issue.

        Set Hitachi8_tInitialised               ' Indicate that the LCD is initialised
        WREG = LCD_WREGStore                    ' Get the saved byte back
        GoTo Hitachi8_Skip                      ' Goto print the data.
Hitachi8_SetCommand:
        Set Hitachi8_tComOrData                 ' Set for command
    EndIf
    '---------------------------------------------------------------------------
Hitachi8_Skip:
    Hitachi8_bByteOut = WREG                    ' Get the byte to send from WREG
    If Hitachi8_tComOrData = 1 Then             ' Command this time?
        Clear __LCD_RSPORT.__LCD_RSPIN          ' Yes. So clear the RS pin
        Sublw 3
        Bnc Hitachi8_SetData                    '  Short delay, Branch If carry = 0 Or >.
        GoSub Hitachi8_SetData                  ' Long delay
        DelayUS __LCD_COMMANDUS                 ' Wait for the command to complete
        Return
    EndIf
    Set Hitachi8_tComOrData                     ' Indicate a command next time
    Sublw 254                                   ' Command next time? (byte is still in WREG)
    Bz Hitachi8_Exit
    Set __LCD_RSPORT.__LCD_RSPIN                ' Set the RS pin

Hitachi8_SetData:
    ' Now send the data byte to the LCD port
    Set __LCD_ENPORT.__LCD_ENPIN                ' Enabled EN pin
    '
':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
$ifdef LCD_Reverse_Bits
    ' Reverse the bits of the port
    PP0 = Hitachi8_bByteOut                     ' Load the byte to be rotated
    WREG = 8                                    ' Create a loop for 8-bits
Hitachi8_RevLoop:
    Ror PP0                                     ' Rotate MSB of PP0 into Carry
    Rol PP0H                                    ' Rotate Carry into LSB of PP0H
    Decfsz WREG,F                               ' \
    Bra Hitachi8_RevLoop                        ' / Do it for all 8-bits
    __LCD_DTPORT = PP0H                         ' Write the byte to the port
$else
    ' Normal byte sent to the port
    __LCD_DTPORT = Hitachi8_bByteOut            ' Write the byte to the port
$endif
':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    DelayCS 6
    Clear Hitachi8_tComOrData                   ' Indicate data next time
    Clear __LCD_ENPORT.__LCD_ENPIN              ' Disabled EN pin
    DelayUS __LCD_DATAUS                        ' Wait for the data operation to complete   
Hitachi8_Exit:
    WREG = LCD_WREGStore                        ' Return the original byte in WREG
    Return
'===============================================================================
#else
'===============================================================================
' Start LCD HITACHI BUS 4 Bits
'    Low __LCD_ENPORT.__LCD_ENPIN                ' Pull the EN pin to output low
'    Low __LCD_RSPORT.__LCD_RSPIN                ' Pull the RS pin to output Low

    ' Set the half LCD PORT to digital.
    #if (__LCD_DTPORT == PORTA)
        #if(__LCD_DTPIN == 0)
            $ifdef _ANSEL
            ANSEL = ANSEL & %11110000
            $endif
            $ifdef _ANSELA
            ANSELA = ANSELA & %11110000
            $endif
        #else
            $ifdef _ANSEL
            ANSEL.5 = 0
            $endif
            $ifdef _ANSELA
            ANSELA.5 = 0
            $endif        
        #endif    
    #endif

    #if (__LCD_DTPORT == PORTB)
        #if(__LCD_DTPIN == 0)
            $ifdef _ANSELH
            ANSELH = ANSELH & %11101000
            $endif
            $ifdef _ANSELB
            ANSELB = ANSELB & %11110000
            $endif    
        #else
            $ifdef _ANSELH
            ANSELH.3 = 0
            $endif
            $ifdef _ANSELB
            ANSELB = ANSELB & %00001111
            $endif    
        #endif 
    #endif

    $ifdef _PORTC
    #if (__LCD_DTPORT == PORTC)
        #if(__LCD_DTPIN == 0)
            $ifdef _ANSELC
            ANSELC = ANSELC & %11110000
            $endif    
        #else
            $ifdef _ANSELC
            ANSELC = ANSELC & %00001111
            $endif    
        #endif  
    #endif
    $endif    

    $ifdef _PORTD
    #if (__LCD_DTPORT == PORTD)
        #if(__LCD_DTPIN == 0)
            $ifdef _ANSELD
            ANSELD = ANSELD & %11110000
            $endif    
        #else
            $ifdef _ANSELD
            ANSELD = ANSELD & %00001111
            $endif    
        #endif 
    #endif
    $endif        
        
    $ifdef _PORTE
    #if (__LCD_DTPORT == PORTE)
        #if(__LCD_DTPIN == 0)
            $ifdef _ANSELH
            ANSELH = ANSELH & %00011111
            $endif
            $ifdef _ANSELE
            ANSELE = ANSELE & %11111000
            $endif
        #endif    
    #endif
    $endif 
    
    ' Set Print PORT to output
    #if(__LCD_DTPIN == 0)
        WREG = $F0
    #else
        WREG = $0F
    #endif
    Andwf (__LCD_DTPORT + 0x12)         ' Set the correct half of the port to output (+18 decimal)

    WREG = LCD_WREGStore                ' Get back the byte. Same as LCD_WREGStore
    '---------------------------------------------------------------------------

    If Hitachi4_tInitialised = 0 Then   ' Has LCD been inititalised?
        DelayMS 150                     ' No. So wait 150ms
        Hitachi4_bByteOut = $30         ' Start Initialise the LCD
        GoSub Hitachi4_NibbleSend       ' Send init
'        DelayUS 2000                    ' Wait 5ms
        DelayUS 5000                    ' Wait 5ms
        GoSub Hitachi4_NibbleSend       ' Send init again
        DelayUS 100                     ' Wait 100us
        GoSub Hitachi4_NibbleSend       ' Send init one more time
        DelayUS 100                     ' Wait 100us
        Hitachi4_bByteOut = $20         ' Function_Set
        GoSub Hitachi4_NibbleSend       ' Send $20
        DelayUS 100                     ' Wait 100us

        WREG = _FUNCTION_SET_4B_2L_T1   ' Function_Set: 4-bit=$29, Table 1
        GoSub Hitachi4_SetCommand       ' 
        WREG = $14                      ' Bias set: BS=0, FX=0   %0001 0100  
        GoSub Hitachi4_SetCommand       ' 
        WREG = $55                      ' Power control: Ion=0, Bon=1, C5=0, C4=1 %0101 0101
        GoSub Hitachi4_SetCommand       ' 
        WREG = $6D                      ' Follower control: Fon=1, Rab2=1, Rab1=0, Rab0=1  %0110 1101
        GoSub Hitachi4_SetCommand       ' 
        '-----------------------------------------------------------------------
        'Setting the LCD Contrast: The contrast value could be written or not in the eeprom memory.
        ' After Reset, the code read the eeprom memory if there is a correct contrast value.
        _mLCD_Index = ERead EEADR_ContrastCode      ' Read the code value saved in the eeprom memory.
        If _mLCD_Index = $5A Then                   ' Check if there is a true contrast value?
            _mLCD_Index = ERead EEADR_ContrastData  ' Yes, there is..., load it.
            _mLCD_Index = _mLCD_Index & 15          ' No more than 15.
            _mContrast = _mLCD_Index                ' Save the value for current library contrast.
            WREG = _mLCD_Index | $70                ' Mix the command with value.
            GoSub Hitachi4_SetCommand               ' Now write the correct value to the LCD.
        Else                                        ' If not contrast written in the eeprom then...
            _mContrast = 7                          ' Load a default contrast.
            WREG = $77                              ' Mix the address...
            GoSub Hitachi4_SetCommand               ' Writing the value to the LCD.
            EWrite EEADR_ContrastData,[_mContrast]  ' It is the default value, write it to the eeprom.
            EWrite EEADR_ContrastCode,[$5A]         ' Write the control too.        
        EndIf
        '-----------------------------------------------------------------------
        #ifdef __LCD_LINES                      ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)               ' Yes. So...
                WREG = _FUNCTION_SET_4B_1L_T0   ' Set for 4-bit mode, 1 line, 5x16 font
            #else
                WREG = _FUNCTION_SET_4B_2L_T0   ' Set for 4-bit mode, 2+ lines, 5x7 font
            #endif
        #else                                   ' Otherwise... Default to 2 lines
            WREG = _FUNCTION_SET_4B_2L_T0       ' Set for 4-bit mode, 2+ lines, 5x7 font
        #endif
        GoSub Hitachi4_SetCommand               ' Write Function Set

        WREG = $0C                              ' %00001100, Display Control: %0000 1100, encender el display
        _DisplayControl = WREG                  ' %00001100, Default value: D=1, C=0, B=0, Display ON, Cursor OFF, Blinking Off.
        GoSub Hitachi4_SetCommand               ' 
        WREG = $01                              ' Clear display
        GoSub Hitachi4_SetCommand               ' 
        WREG = $06                              ' %00000110, Entry Mode set, Shift cursor to right when printing. 
        GoSub Hitachi4_SetCommand               ' 
        WREG = $02                              ' Return Home. Set DDRAM address to "00H" from AC and return cursor to its original
        GoSub Hitachi4_SetCommand               ' position if shifted. The contents of DDRAM are not changed.
                                                ' Sometimes the ST7036 is sensitive with bad reset, then this command saves this issue.

        Set Hitachi4_tInitialised               ' Indicate that the LCD is initialised
        WREG = LCD_WREGStore                    ' Get the saved byte back
        GoTo Hitachi4_Skip                      ' Goto print the data.
Hitachi4_SetCommand:
        Set Hitachi4_tComOrData                 ' Set for command
    EndIf
    '---------------------------------------------------------------------------
Hitachi4_Skip:
    Hitachi4_bByteOut = WREG            ' Get the byte to send from WREG
    If Hitachi4_tComOrData = 1 Then     ' Is it a command this time?
        Clear __LCD_RSPORT.__LCD_RSPIN  ' Yes. So set command register select
        Sublw 3
        Bnc Hitachi4_SetData            ' Short delay, Branch if carry = 0 or >.
        GoSub Hitachi4_SetData          ' Long delay
        DelayUS __LCD_COMMANDUS         ' Wait for the command to complete       
        Return
    EndIf
    Set Hitachi4_tComOrData             ' Indicate first nibble, or command next time
    Sublw $FE                           ' Command next time? (the byte is still in WREG)
    Bz Hitachi4_Exit                    ' Exit the print routine.
    Set __LCD_RSPORT.__LCD_RSPIN        ' Set the RS pin

Hitachi4_SetData:
    ' The original code dosen't work with (__LCD_DTPIN == 0) with bits reversed.
    $ifndef LCD_Reverse_Bits
    Set_Bank Hitachi4_bByteOut
    #if(__LCD_DTPIN == 0)               ' PORT in Low nibble?
        Swapf Hitachi4_bByteOut,F       ' Swap top and bottom nibbles. Error for Reversed bits in PORTX.0.
    #endif
    $endif
    Btfss Hitachi4_tComOrData,0         ' First time through only

Hitachi4_NibbleSend:
    Clear Hitachi4_tComOrData           ' Indicate second nibble, and data next time
    #if(__LCD_DTPIN == 0)               ' Prepare the LCD PORT.
        WREG = $F0                      ' Clear the bits of the LCD data port
    #else
        WREG = $0F                      ' Clear the bits of the LCD data port
    #endif
    ' Erase the Print PORT.
    Set_Bank __LCD_DTPORT
    Andwf __LCD_DTPORT,F
    DelayCS 6
    Set __LCD_ENPORT.__LCD_ENPIN        ' Enabled EN pin
    '
':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
$ifdef LCD_Reverse_Bits
    ' Reverse the bits of the port
    PP0 = Hitachi4_bByteOut             ' Load the byte to be rotated
    #if(__LCD_DTPIN == 0)
        WREG = 8                        ' Create a loop for 4-bits low nibble: 8 times (12 if do not swaf the data before)
Hitachi4_RevLoop:
        Rol PP0                         ' Rotate MSB of PP0 to the left into the Carry
        Ror PP0H                        ' Rotate the Carry to the right into LSB of PP0H
        Decfsz WREG,F                   ' \
        Bra Hitachi4_RevLoop            ' / Do it for all 8-bits
    #else
        WREG = 4                        ' Create a loop for 4-bits high nible: 4 times
Hitachi4_RevLoop2:
        Rol PP0                         ' Rotate MSB of PP0 into the Carry
        Ror PP0H                        ' Rotate the Carry into LSB of PP0H
        Decfsz WREG,F                   ' \
        Bra Hitachi4_RevLoop2           ' / Do it for all 4-bits
    #endif
    WREG = PP0H                         ' for reverse bits. Isolate correct nibble of the byte
$else
    ' Normal byte sent to the port
    WREG = Hitachi4_bByteOut            ' Load the byte.
$endif
':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
' Now prepare the data to print to the PORT
    ' Data is in WREG.
    #if(__LCD_DTPIN == 0)
        Andlw $0F
    #else
        Andlw $F0
    #endif
    Set_Bank __LCD_DTPORT
    Iorwf __LCD_DTPORT,F                        ' Write the byte to the port
    DelayCS 6
    Clear __LCD_ENPORT.__LCD_ENPIN              ' Disabled EN pin. Data has been written to the LCD PORT.
    Set_Bank Hitachi4_bByteOut
    Swapf Hitachi4_bByteOut,F                   ' Move to the other nibble
    DelayUS __LCD_DATAUS                        ' Wait for the data operation to complete for every nibble.  
    Set_Bank Hitachi4_tComOrData
    Btfsc Hitachi4_tComOrData,0
    Bra Hitachi4_NibbleSend
Hitachi4_Exit:
    WREG = LCD_WREGStore                        ' Return the original byte in WREG
    Return
#endif      ' End LCD BUS 4 Bits
'===============================================================================
'-------------------------------------------------------------------------------
' Function Name: Write the Function Set with the Bit For Table0
' Input        : none
' Output       : none
' Syntax       : LCD_FunctionSet_TBL0()
' Notes        : 
'
'-------------------------------------------------------------------------------
'   FUNCTION SET
'-------------------------------------------------------------------------------
'   0   0   0   0   1   DL  N  DH  IS2 IS1
'   RS R/W DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
'
'   *DL : Interface data length control bit
'       When DL = "High", it means 8-bit bus mode with MPU.
'       When DL = "Low", it means 4-bit bus mode with MPU. So to speak,
'       DL is a signal to select 8-bit or 4-bit bus mode.
'       When 4-bit bus mode, it needs to transfer 4-bit data by two times.
'   *N : Display line number control bit
'       When N = "High", 2-line display mode is set.
'       When N = "Low", it means 1-line display mode.
'       When N3 option pin connect to VDD, N must set N=1.
'   *DH : Double height font type control bit
'       When DH = " High " and N= Low, display font is selected to double height
'       mode(5x16 dot),RAM address can only use 00H~27H.
'       When DH= High and N= High, it is forbidden.
'       When DH = " Low ", display font is normal (5x8 dot).
'-------------------------------------------------------------------------------
    
$define LCD_FunctionSet_TBL0() mFunctionSet_TBL0

mFunctionSet_TBL0 Macro-
    GoSub mFunctionSet_TBL0_Sub
Endm
'
#ifMacro- mFunctionSet_TBL0
mFunctionSet_TBL0_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    #if (__LCD_INTERFACE == 4)                      ' Has the LCD_Bus 4 Bits?
        #ifdef __LCD_LINES                          ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)                   ' Yes. So...
                Print $FE, _FUNCTION_SET_4B_1L_T0   ' Set the page 0, 1 line
            #else
                Print $FE, _FUNCTION_SET_4B_2L_T0   ' Set the page 0, 2 lines
            #endif
        #else                                       ' Otherwise... Default to 2 lines
            Print $FE, _FUNCTION_SET_4B_2L_T0       ' Set the page 0, 2 lines
        #endif
    #else                                           ' Has the LCD_Bus 8 Bits?
        #ifdef __LCD_LINES                          ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)                   ' Yes. So...
                Print $FE, _FUNCTION_SET_8B_1L_T0   ' Set the page 0, 1 line
            #else
                Print $FE, _FUNCTION_SET_8B_2L_T0   ' Set the page 0, 2 lines
            #endif
        #else                                       ' Otherwise... Default to 2 lines
            Print $FE, _FUNCTION_SET_8B_2L_T0       ' Set the page 0, 2 lines
        #endif
    #endif
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Write the Function Set with the Bit For Table1
' Input        : none
' Output       : none
' Syntax       : LCD_FunctionSet_TBL1()
' Notes        : Change the LCD to table 1 for writing.
'    
$define LCD_FunctionSet_TBL1() mFunctionSet_TBL1

mFunctionSet_TBL1 Macro-
    GoSub mFunctionSet_TBL1_Sub
Endm

#ifMacro- mFunctionSet_TBL1
mFunctionSet_TBL1_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    #if (__LCD_INTERFACE == 4)                      ' Has the LCD_Bus 4 Bits?
        #ifdef __LCD_LINES                          ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)                   ' Yes. So...
                Print $FE, _FUNCTION_SET_4B_1L_T1   ' Set the page 1, 1 line
            #else
                Print $FE, _FUNCTION_SET_4B_2L_T1   ' Set the page 1, 2 lines
            #endif
        #else                                       ' Otherwise... Default to 2 lines
            Print $FE, _FUNCTION_SET_4B_2L_T1       ' Set the page 1, 2 lines
        #endif
    #else                                           ' Has the LCD_Bus 8 Bits?
        #ifdef __LCD_LINES                          ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)                   ' Yes. So...
                Print $FE, _FUNCTION_SET_8B_1L_T1   ' Set the page 1, 1 line
            #else
                Print $FE, _FUNCTION_SET_8B_2L_T1   ' Set the page 1, 2 lines
            #endif
        #else                                       ' Otherwise... Default to 2 lines
            Print $FE, _FUNCTION_SET_8B_2L_T1       ' Set the page 1, 2 lines
        #endif
    #endif
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Write the Function Set with the bit for Table2
' Input        : 
' Output       : 
' Syntax       : LCD_FunctionSet_TBL2()
' Notes        : 
'    
$define LCD_FunctionSet_TBL2() mFunctionSet_TBL2

mFunctionSet_TBL2 Macro-
    GoSub mFunctionSet_TBL2_Sub
Endm

#ifMacro- mFunctionSet_TBL2
mFunctionSet_TBL2_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    #if (__LCD_INTERFACE == 4)                      ' Has the LCD_Bus 4 Bits?
        #ifdef __LCD_LINES                          ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)                   ' Yes. So...
                Print $FE, _FUNCTION_SET_4B_1L_T2   ' Set the page 2, 1 line
            #else
                Print $FE, _FUNCTION_SET_4B_2L_T2   ' Set the page 2, 2 lines
            #endif
        #else                                       ' Otherwise... Default to 2 lines
            Print $FE, _FUNCTION_SET_4B_2L_T2       ' Set the page 2, 2 lines
        #endif
    #else                                           ' Has the LCD_Bus 8 Bits?
        #ifdef __LCD_LINES                          ' Has the LCD_Lines declare been used in the main program?
            #if(__LCD_LINES == 1)                   ' Yes. So...
                Print $FE, _FUNCTION_SET_8B_1L_T2   ' Set the page 2, 1 line
            #else
                Print $FE, _FUNCTION_SET_8B_2L_T2   ' Set the page 2, 2 lines
            #endif
        #else                                       ' Otherwise... Default to 2 lines
            Print $FE, _FUNCTION_SET_8B_2L_T2       ' Set the page 2, 2 lines
        #endif
    #endif
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Return the current contrast loaded with the library code.
' Input        : None
' Output       : Return Byte
' Syntax       : Var = LCD_ReadContrast()  
' Notes        : 
'
$define LCD_ReadContrast() _mRead_Contrast

_mRead_Contrast Macro-
    #if (_mRead_Contrast_RETURN != 1)
        #error "_mRead_Contrast - Mandatory return parameter misssing"
    #else
        #if(Return_Type != Byte) && (Return_Type != Word)
            #error "_mRead_Contrast - Return variable should be a Byte or Word variable"
        #else
            #if (Return_Type == Word)
                Byte_Word _mContrast, Return_Var
            #endif            
            #if (Return_Type == Byte)
                Byte_Byte _mContrast, Return_Var
            #endif
        #endif
    #endif
Endm
'-------------------------------------------------------------------------------
' Function Name: Save the _mContrast value to the LCD
' Input        : 
' Output       : 
' Syntax       : Save_Contrast(pValue)
' Notes        : write the _mContrast value to the LCD
'
'    0   1   1   1   C3  C2  C1  C0
'   DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
'
'   C3,C2,C1,C0:_mContrast set(low byte)
'       C5,C4,C3,C2,C1,C0 can only be set when internal follower is used (OPF1=0,OPF2=0).
'       They can more precisely adjust the input reference voltage of V0 generator.
'       The details please refer to the supply voltage for LCD driver.

$define LCD_WriteContrast(pValue) mWriteContrast pValue

mWriteContrast Macro- pValue
    #if (Prm_Count > 1)
   	    #error "LCD_WriteContrast - Too many parameters"
   	    Exitm
    #else
    	#if (Prm_Count < 1)
    		#error "LCD_WriteContrast - Too few parameters"
    	    Exitm
        #else
    		#if (Prm_1 != Num8) && (Prm_1 != Byte)
    			#error "LCD_WriteContrast Macro - (Param1) Should be a Num8 or a Byte"
    		    Exitm
            #else
                #if (Prm_1 == Byte) 
                    Byte_Byte pValue, _mLCD_Index 
                #endif
                #if (Prm_1 == Num8) 
                    Num_Byte pValue, _mLCD_Index 
                #endif
                GoSub mWrite_Contrast_Sub
            #endif
        #endif            
    #endif
Endm
 
#ifMacro- mWriteContrast
mWrite_Contrast_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    _mLCD_Index = _mLCD_Index & $0F
    If _mLCD_Index <> _mContrast Then
        _mContrast = _mLCD_Index        
        LCD_FunctionSet_TBL1()                  ' Set Table 1.
         _mLCD_Index = _mContrast | $70         ' Mix the address and data.
        Print $FE, _mLCD_Index                  ' Write the contrast value to the LCD.    
        LCD_FunctionSet_TBL0()                  ' Come back Table 0.
    EndIf
    Return
#endIfMacro-

'-------------------------------------------------------------------------------
' Function Name: Write the current Contrast value to the eeprom data
' Input        : 
' Output       : Return the value read from the eeprom.
' Syntax       : Control = Save_Contrast()
' Notes        : The eeprom memory is read after write and returns the value.
'
$define LCD_SaveContrast() mSave_Contrast

mSave_Contrast Macro-
    GoSub mSave_Contrast_Sub
    #if (mSave_Contrast_RETURN == 1)
        #if (Return_Type == Byte)
            Byte_Byte _mContrast, Return_Var
        #endif
    #endif
Endm
 
'#ifMacro- mSave_Contrast
'mSave_Contrast_Sub:
'        WREG = ERead EEADR_ContrastData             ' read the current value.
'        If WREG <> _mLCD_Index Then                 ' If there is a different then write.
'            EWrite EEADR_ContrastData,[_mContrast]  ' Write the new data 
'            EWrite EEADR_ContrastCode,[$5A]         ' and re-write the code.
'        EndIf
'        _mLCD_Index = ERead EEADR_ContrastData      ' Read the new data for control.
'    Return
'#endIfMacro-

'#ifMacro- mSave_Contrast
'mSave_Contrast_Sub:
'        WREG = ERead EEADR_ContrastData             ' read the current value.
'        If WREG <> _mContrast Then                  ' If there is a different then write.
'            EWrite EEADR_ContrastData,[_mContrast]  ' Write the new data 
'            EWrite EEADR_ContrastCode,[$5A]         ' and re-write the code.
'        EndIf
'        _mContrast = ERead EEADR_ContrastData      ' Read the new data for control.
'    Return
'#endIfMacro-
#ifMacro- mSave_Contrast
mSave_Contrast_Sub:
        WREG = ERead EEADR_ContrastData             ' read the current value.
        If WREG <> _mContrast Then                  ' If there is a different then write.
            EWrite EEADR_ContrastData,[_mContrast]  ' Write the new data 
            EWrite EEADR_ContrastCode,[$5A]         ' and re-write the code.
            DelayMS 5
            _mContrast = ERead EEADR_ContrastData   ' Read the new data for control.
        EndIf
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Clear the LCD data. (2 lines)
' Input        : none
' Output       : none
' Syntax       : LCD_Clear()
' Notes        : This command need 2 mS delay.
'
$define LCD_Clear() _mLCD_Clear

_mLCD_Clear Macro-
    GoSub _mLCD_Clear_Sub
Endm
 
#ifMacro- _mLCD_Clear
_mLCD_Clear_Sub:
    Print $FE, csLCD_ClearDisplay
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Return the cursor to home.
' Input        : none
' Output       : none
' Syntax       : LCD_ReturnHome()
' Notes        : This command need 2 mS delay.
'
$define LCD_ReturnHome() _mLCD_ReturnHome

_mLCD_ReturnHome Macro-
    GoSub _mLCD_ReturnHome_Sub
Endm
 
#ifMacro- _mLCD_ReturnHome
_mLCD_ReturnHome_Sub:
    Print $FE, csLCD_ReturnHome
    Return
#endIfMacro-

'-------------------------------------------------------------------------------
' Function Name: Turn the display On
' Input        : none
' Output       : none
' Syntax       : LCD_On()
' Notes        : In the _DisplayControl variable the command is included, take care don't clear it.
'
'    0   0   0   0   1   D   C   B
'   DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
'
'   Control display/cursor/blink ON/OFF 1 bit register.
'   *D : Display ON/OFF control bit
'       When D = "High", entire display is turned on.
'       When D = "Low", display is turned off, but display data is remained in DDRAM.
'   *C : Cursor ON/OFF control bit
'       When C = "High", cursor is turned on.
'       When C = "Low", cursor is disappeared in current display, but I/D register
'       remains its data.
'   *B : Cursor Blink ON/OFF control bit
'       When B = "High", cursor blink is on, that performs alternate between all
'       the high data and display character at the cursor position.
'       When B = "Low", blink is off.
'   Blinking cursor every 64 frames.
 
$define LCD_On() _mDisplay_On

_mDisplay_On Macro-
    GoSub mDisplay_On_Sub
Endm

#ifMacro- _mDisplay_On
mDisplay_On_Sub:
    _DisplayControl = _DisplayControl | sLCD_Display_ON
    Print $FE, _DisplayControl
    Return    
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: LCD_Off
' Input        : none
' Output       : none
' Syntax       : LCD_Off()
' Notes        : In the _DisplayControl Variable the command is included, take care and don't clear it.
'   
$define LCD_Off() _mNO_Display

_mNO_Display Macro-
    GoSub mNO_Display_Sub
Endm

#ifMacro- _mNO_Display
mNO_Display_Sub:
    _DisplayControl = _DisplayControl & sLCD_Display_OFF
    Print $FE, _DisplayControl
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Turns the underline Cursor On
' Input        : none
' Output       : none
' Syntax       : LCD_Cursor_On()
' Notes        : 
'
$define LCD_Cursor_On() mCursor_On

mCursor_On Macro-
    GoSub mCursor_On_Sub
Endm

#ifMacro- mCursor_On
mCursor_On_Sub:
    _DisplayControl = _DisplayControl | sLCD_Cursor_ON
    Print $FE, _DisplayControl
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: The cursor is disabled.
' Input        : none
' Output       : none
' Syntax       : LCD_Cursor_Off()
' Notes        : 
'            
$define LCD_Cursor_Off() mNO_Cursor

mNO_Cursor Macro-
    GoSub mNO_Cursor_Sub
Endm

#ifMacro- mNO_Cursor
mNO_Cursor_Sub:
    _DisplayControl = _DisplayControl & sLCD_Cursor_OFF
    Print $FE, _DisplayControl
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Turn Cursor to blink On
' Input        : none
' Output       : none
' Syntax       : LCD_Cursor_BlinkOn()
' Notes        : 
'
$define LCD_Cursor_BlinkOn() mCursor_Blinking_On
            
mCursor_Blinking_On Macro-
    GoSub mCursor_Blinking_On_Sub
Endm

#ifMacro- mCursor_Blinking_On
mCursor_Blinking_On_Sub:
    _DisplayControl = _DisplayControl | sLCD_Blink_ON
    Print $FE, _DisplayControl
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Turn the Cursor To blink Off
' Input        : none
' Output       : none
' Syntax       : LCD_Cursor_BlinkOff()
' Notes        : 
'
$define LCD_Cursor_BlinkOff() mNO_Blink        

mNO_Blink Macro-
    GoSub mNO_Blink_Sub
Endm

#ifMacro- mNO_Blink
mNO_Blink_Sub:
    _DisplayControl = _DisplayControl & sLCD_Blink_OFF
    Print $FE, _DisplayControl
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Turn the cursor OFF and the Blinking OFF
' Input        : none
' Output       : none
' Syntax       : LCD_NOCursor()
' Notes        :
'        
$define LCD_NOCursor() _mAllCursor_Off

_mAllCursor_Off Macro-
    GoSub _mAllCursor_Off_Sub
Endm

#ifMacro- _mAllCursor_Off
_mAllCursor_Off_Sub:
    _DisplayControl = csLCD_ControlDisp_ON
    Print $FE, _DisplayControl
    Return
#endIfMacro-
'===============================================================================
'-------------------------------------------------------------------------------
' Function Name: Point the Address in CGRAM
' Input        : none
' Output       : none
' Syntax       : LCD_Set_1stAddress_CGRAM()
' Notes        : IS = 0, Runs "FunctionSet_Basic()" first
'              : Set the command Mode to point to the fisrt byte Address of CGRAM
'
'   Set CGRAM Address
'    0   1  AC5 AC4 AC3 AC2 AC1 AC0
'   DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
'
'   Set CGRAM address to AC.
'   This instruction makes CGRAM data available from MPU.

$define LCD_Set_Address_CGRAM(pValue) mSet_Address_CGRAM pValue

mSet_Address_CGRAM Macro- pValue
    #if (Prm_Count > 1)
	   #error "mSet_Address_CGRAM - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "mSet_Address_CGRAM - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Byte) && (Prm_1 != Num8)
                #error "mSet_Address_CGRAM - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if (Prm_1 == Byte) 
                    Byte_Byte pValue, _mLCD_Index         
                #endif
                #if (Prm_1 == Num8) 
                    Num_Byte pValue, _mLCD_Index         
                #endif
                GoSub _mSet_Address_CGRAM_Sub
            #endif
        #endif
    #endif
Endm

#ifMacro- mSet_Address_CGRAM
_mSet_Address_CGRAM_Sub:
'    LCD_FunctionSet_TBL0()                  ' Set table 0
    _mLCD_Index = (_mLCD_Index & %00111111) | csLCD_CGRAM_ADDR0
    Print $FE, _mLCD_Index
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
' Function Name: Point the Address in CGRAM0
' Input        : None
' Output       : None
' Syntax       : LCD_Set_Address_CGRAM0()
' Notes        : IS = 0, Runs "FunctionSet_Basic()" first
'              : Set the command Mode to point to the fisrt byte Address of CGRAM
'
' Set DDRAM address to AC.
' This instruction makes DDRAM data available from MPU.
' When 1-line display mode (N = 0), DDRAM address is from "00H" to "4FH".
' In 2-line display mode (N = 1), DDRAM address in the 1st line is from "00H" to "27H", and
' DDRAM address in the 2nd line is from "40H" to "67H".
' In 3-line display mode (N3=1, N=1), DDRAM address in the 1st line is from 00H to OFH, DDRAM in the
' 2nd line is from 10H to 1FH, and DDRAM in the 3rd line is from 20H to 2FH.
'
$define LCD_Set_1stAddress_CGRAM() mSet_Address_CGRAM0

$define LCD_Set_Address_CGRAM0() mSet_Address_CGRAM0

mSet_Address_CGRAM0 Macro-
    GoSub _mSet_Address_CGRAM0_Sub
Endm

#ifMacro- mSet_Address_CGRAM0
_mSet_Address_CGRAM0_Sub:
'    LCD_FunctionSet_TBL0()          ' Set table 0
'    _mLCD_Index = csLCD_CGRAM_ADDR0
'    Print $FE, _mLCD_Index
    Print $FE, csLCD_CGRAM_ADDR0
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Clear the CGRAM data
' Input        : None
' Output       : None
' Syntax       : LCD_Clear_CGRAM()
' Notes        : 
'
$define LCD_Clear_CGRAM() _mClear_CGRAM

_mClear_CGRAM Macro-
    GoSub _mClear_CGRAM_Sub
Endm

#ifMacro- _mClear_CGRAM
_mClear_CGRAM_Sub:
    LCD_Set_Address_CGRAM0()    ' Command, Point to the first address of CGRAM
    _mLCD_Index = 0
    Repeat
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Print $00               ' Character 0: Blank
        Inc _mLCD_Index         ' Next byte to clear
    Until _mLCD_Index > 63      ' Print Character '0' 64 times
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Point the Address in DDRAM
' Input        : None
' Output       : None
' Syntax       : LCD_Set_Address_DDRAM(Address)
' Notes        : Set the command Mode to point to the fisrt byte Address of DDRAM
'
'   Set DDRAM Address
'    1  AC5  AC5 AC4 AC3 AC2 AC1 AC0
'   DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
'
'   Set DDRAM address to AC.
'   This instruction makes DDRAM data available from MPU.

$define LCD_Set_Address_DDRAM(pValue) _mSet_Address_DDRAM pValue

_mSet_Address_DDRAM Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mSet_Address_DDRAM - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mSet_Address_DDRAM - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Byte) && (Prm_1 != Num8)
                #error "_mSet_Address_DDRAM - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if (Prm_1 == Byte) 
                    Byte_Byte pValue, _mLCD_Index         
                #endif
                #if (Prm_1 == Num8) 
                    Num_Byte pValue, _mLCD_Index         
                #endif
                GoSub _mSet_Address_DDRAM_Sub
            #endif
        #endif
    #endif
Endm

#ifMacro- _mSet_Address_DDRAM
_mSet_Address_DDRAM_Sub:
'    LCD_FunctionSet_TBL0()                  ' Set table 0
    _mLCD_Index = _mLCD_Index | cLCD_SetDDRAMADDR
    Print $FE, _mLCD_Index
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Cursor and Display Shift
'-------------------------------------------------------------------------------
' Function Name: scroll Cursor Left 
' Input        : None
' Output       : None
' Syntax       : LCD_MoveCursorToLeft()
' Notes        : Table 0
' These commands scroll the display without changing the RAM

' Cursor or Display Shift:
'    0   0   0   1   S/C  R/L  X   X
'   DB7 DB6 DB5 DB4  DB3  DB2 DB1 DB0
'
' S/C: Screen/Cursor select bit
'   When S/C=High, Screen is controlled by R/L bit.
'   When S/C=Low, Cursor is controlled by R/L bit.
'
' R/L: Right/Left
'   When R/L=High, set direction to right.
'   When R/L=Low, set direction to left.
'
' Without writing or reading of display data, shift right/left cursor position or display.
' This instruction is used to correct or search display data. During 2-line mode display,
' cursor moves to the 2nd line after 40th digit of 1st line.
' Note that display shift is performed simultaneously in all the line.
' When displayed data is shifted repeatedly, each line shifted individually.
' When display shift is performed, the contents of address counter are not changed.
'
'   S/C  R/L    Description                                                  AC Value
'    L    L     Shift cursor to the left                                     AC=AC-1
'    L    H     Shift cursor to the right                                    AC=AC+1
'    H    L     Shift display to the left. Cursor follows the display shift  AC=AC
'    H    H     Shift display to the right. Cursor follows the display shift AC=AC

$define LCD_MoveCursorToLeft() mMoveCursorLeft

mMoveCursorLeft Macro-
    GoSub mScrollCursorLeft_Sub
Endm

#ifMacro- mMoveCursorLeft   
mScrollCursorLeft_Sub:
'    LCD_FunctionSet_TBL0()                       ' Set the table 0 fisrt
    Print $FE, csLCD_CursorToLeft
    Return 
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: scroll Cursor Right 
' Input        : None
' Output       : None
' Syntax       : LCD_MoveCursorToRight()
' Notes        : 
' 
$define LCD_MoveCursorToRight() mMoveCursorRight

mMoveCursorRight Macro-
    GoSub mScrollCursorRight_Sub
Endm

#ifMacro- mMoveCursorRight    
mScrollCursorRight_Sub:
'    LCD_FunctionSet_TBL0()                       ' Set the table 0 fisrt
    Print $FE, csLCD_CursorToRight
    Return 
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Scroll the Display to Left 
' Input        : None
' Output       : None
' Syntax       : LCD_Scroll_Display_Left()
' Notes        :
'
$define LCD_ScrollDisplayToLeft() mScrollDisplayLeft

mScrollDisplayLeft Macro-
    GoSub mScrollDisplayLeft_Sub
Endm

#ifMacro- mScrollDisplayLeft    
mScrollDisplayLeft_Sub:
'    LCD_FunctionSet_TBL0()                      ' Set the table 0 fisrt
    Print $FE, csLCD_DisplayToLeft
    Return 
#endIfMacro-
'-------------------------------------------------------------------------------
' Function: Scroll the Display to Right
' Input   : None
' Output  : None
' Syntax  : LCD_Scroll_Display_Right()
' Notes   :
'
$define LCD_ScrollDisplayToRight() mScrollDisplayRight

mScrollDisplayRight Macro-
    GoSub mScrollDisplayRight_Sub
Endm

#ifMacro- mScrollDisplayRight
mScrollDisplayRight_Sub:
'    LCD_FunctionSet_TBL0()                      ' Set the table 0 fisrt
    Print $FE, csLCD_DisplayToRight
    Return  
#endIfMacro-
'===============================================================================
' START: Entry Mode Set
'-------------------------------------------------------------------------------
' Function Name: LCD_DisplayNOShift()
' Input        : None
' Output       : None
' Syntax       : LCD_DisplayNOShift()
' Notes        : Entry Mode Set
'              : The LCD_Return_Home() realizes the same function when
'              : the display is shifted.
'
' Entry Mode Set
'    0   0   0   0   0   1  I/D  S
'   DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0

' Set the moving direction of cursor and display.
' I/D : Increment / decrement of DDRAM address (cursor or blink)
'   When I/D = "High", cursor/blink moves to right and DDRAM address is increased by 1.
'   When I/D = "Low", cursor/blink moves to left and DDRAM address is decreased by 1.
'
'* CGRAM operates the same as DDRAM, when read from or write to CGRAM.
' S: Shift of entire display
'   When DDRAM read (CGRAM read/write) operation or S = "Low", shift of entire display
'   is not performed.
'   If S = "High" and DDRAM write operation, shift of entire display is performed
'   according to I/D value (I/D = "1" : shift left, I/D = "0" : shift right).
'
'   I/D  S   Description
'   H   H   Shift the display to the left
'   L   H   Shift the display to the right
'
$define LCD_DisplayNOShift() mDisplayNOShift

mDisplayNOShift Macro-
    GoSub mDisplayNOShift_Sub
Endm

#ifMacro- mDisplayNOShift
mDisplayNOShift_Sub:
    Print $FE, csLCD_EntryModeSetNOShift
    Return  
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name: Move the Display_To_Left when printing
' Input        : None
' Output       : None
' Syntax       : LCD_DisplayToLeft()
' Notes        : Entry Mode Set
'
$define LCD_DisplayToLeft() mDisplayToLeft

mDisplayToLeft Macro-
    GoSub mDisplayToLeft_Sub
Endm

#ifMacro- mDisplayToLeft
mDisplayToLeft_Sub:
    Print $FE, csLCD_EntryMSShiftDisplayToLeft
    Return  
#endIfMacro-
'-------------------------------------------------------------------------------
' Function Name     : Display_To_Right when printing
' Input Variables   : 
' Output Variables  : 
' Syntax            : LCD_DisplayToRight()
' Notes             : Entry Mode Set
'
$define LCD_DisplayToRight() mDisplayToRight

mDisplayToRight Macro-
    GoSub mDisplayToRight_Sub
Endm

#ifMacro- mDisplayToRight   
mDisplayToRight_Sub:    
    Print $FE, csLCD_EntryMSShiftDisplayToRight
    Return  
#endIfMacro-
'-------------------------------------------------------------------------------
' END: Entry Mode Set
'===============================================================================
' Function Name: Clear the line 1 of the LCD.
' Input        : number of line
' Output       : none
' Syntax       : LCD_Clear_Row(Number Line)
' Notes        : 
' Write a "SPACE" to the LCD column 1 to column 16, Row1, and place the cursor in column1, Row1.

    Dim _xNbLines As LVSByte1

$define LCD_Clear_Row(pValue) mClear_Row pValue

mClear_Row Macro- pValue
    #if (Prm_Count > 1)
   	    #error "LCD_Clear_Row - Too many parameters"
   	    Exitm
    #else
    	#if (Prm_Count < 1)
    		#error "LCD_Clear_Row - Too few parameters"
    	    Exitm
        #else
    		#if (Prm_1 != Num8)
    			#error "LCD_WriteContrast Macro - (Param1) Should be a Num8"
    		    Exitm
            #else
                #ifdef __LCD_LINES
                    #if (pValue >__LCD_LINES)
                        #error "There is NO such line in this LCD"
                        Exitm
                    #else
                        Num_Byte pValue, _xNbLines
                    #endif
                    GoSub mClear_Row_Sub  
                #else
                    #error "The number of lines is not declared"
                    Exitm                
                #endif                    
            #endif
        #endif            
    #endif
Endm

#ifMacro- mClear_Row
mClear_Row_Sub:
    Cursor _xNbLines,1          ' Set the Cursor to the origin.
    _mLCD_Index = 0
    Repeat
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Print $20               ' Print a Blank
        Inc _mLCD_Index         ' Increment the index
    Until _mLCD_Index > 15  
    Cursor _xNbLines,1          ' Set the Cursor to the origin.
    Return
#endIfMacro-        
'-------------------------------------------------------------------------------
' Function Name: Alternative To the standard Proton Cls command
' Input        : Line. Position 
' Output       : none
' Syntax       : CLL {Line, {Pos}} Command
' Notes        : Helps to clear some characters of the LCD.
'
' For users still using an alphanumeric LCD this provides a useful enhanced
' alternative to the standard Proton CLS command.  When invoked with no parameters
' it will perform a standard Proton CLS, however, if you add a line number argument
' it will clear just that line, if you add a second argument (Position) it will clear
' from that position to the end of the line leaving the cursor at the beginning of the clear.
'
' CLL {Line, {Pos}} Command
'
'      With no arguments same as proton CLS
'
'      With 1 argument (Line), clears line specified in argument
'      Con 1 argumento (Linea), borra la lnea especificada en el argumento
'
'      With 2 arguments (Line, Pos), clears to end of Line from Position
'      Con 2 argumentos (Linea, Posicin), borra el final de la lnea desde el argumento Posicin
'
$ifndef CLL
    Dim xCUR_Row As LVSByte1
    Dim xCUR_Col As LVSByte2
    Dim xCUR_Chrs As LVSByte3
    
$define CLL(pLine, pPos) _mCLL pLine, pPos
 
_mCLL Macro- pLine, pPos
    #if (Prm_Count >  2)
        #error "_mCLL - Too many parameters"
    #else
        #if (Prm_Count == 0)
'      Add additional code for no params here .......
        Cls    ' Here is the additional code added after the generator has produced the outline
        #else
            ' 1 additional character: Delete the line x. (all characters)
            #if (Prm_Count == 1)
                #if (Prm_1 != Byte) && (Prm_1 != Num8)
                    #error "_mCLL - Line(Param 1) should be a Byte variable or number"
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pLine, xCUR_Row
                #endif
                #if (Prm_1 == Num8)
                    Num_Byte pLine, xCUR_Row
                #endif
                GoSub _mCLL_Sub1
            #endif

            ' 2 additional characters: Delete the line x from position y.
            #if (Prm_Count == 2)
                #if (Prm_1 != Byte) && (Prm_1 != Num8)
                    #error "_mCLL - Line(Param 1) should be a Byte variable or number"
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pLine, xCUR_Row
                #endif
                #if (Prm_1 == Num8)
                    Num_Byte pLine, xCUR_Row
                #endif

                #if (Prm_2 != Byte) && (Prm_2 != Num8)
                    #error "_mCLL - Pos(Param 2) should be a Byte variable or number"
                #endif
                #if (Prm_2 == Byte)
                    Byte_Byte pPos, xCUR_Col
                #endif
                #if (Prm_2 == Num8)
                    Num_Byte pPos, xCUR_Col
                #endif
                GoSub _mCLL_Sub2
            #endif

        #endif
    #endif
Endm

#ifMacro- _mCLL
_mCLL_Sub1:
    xCUR_Col = 1                ' The Column most be > 0.

_mCLL_Sub2:
    Cursor xCUR_Row, xCUR_Col 
    xCUR_Chrs = __LCD_CharsLenght - xCUR_Col
    Inc xCUR_Chrs
    Repeat
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Print $20               ' Send a SPACE to the LCD
        Dec xCUR_Chrs           ' More character to delete.
    Until xCUR_Chrs = 0         ' Until the last number of characters deleted
    Cursor xCUR_Row, xCUR_Col   ' Set the Cursor to the Origin position.
    Return
#endIfMacro-
$endif
'Note  This macro requires a constant LCD_CharsLenght to be defined to reflect your
' hardware configuration.  E.g. Symbol LCD_CharsLenght = 16 for a 16 characters display.  
'===============================================================================
$ifdef _HRSOutVersionFile_inc_
_ST7036_INC_Version: CData "LCD_ST7036.inc Version 1.0 - 17/01/2016",0
$endif
'===============================================================================
OverST7036_Lib:
$endif  ' _ST7036_INC_   

