'****************************************************************
'*  Name    : ORG_SyncBlock_K20.inc                             *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 24/03/2017                                        *
'*  Version : 1.0                                               *
'*  Notes   : Syncronize the ORG command withe the head of a    *
'*          : Write or Erase Block for PICs K20 series.         *
'****************************************************************
'
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es un software libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' trs aprci.
'-------------------------------------------------------------------------------
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
' PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHOR OR COPYRIGHT
' HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
' OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'###############################################################################
'
(*
    From the PDS Manual 
    Syntax
    Org Value
    Overview
    Set the program origin for subsequent code at the address defined in Value
    Operators
    Value can be any constant value within the range of the particular microcontroller's memory.
    Example:
    Org 2000 ' Set the origin to address 2000

    This Macro syncronizes every ORG command with the Head of the Erase or Write Block.
    Using: ORG_EraseBlock(1000)
    Using: ORG_WriteBlock(1000)    
'-------------------------------------------------------------------------------
    Before the file write the Erase Block of youe PIC. eg:

    $define ERASE_FLASH_BLOCKSIZE 64 
    Include "ORG_SyncBlock.Inc"
*)
'-------------------------------------------------------------------------------
    $ifndef _ORG_SyncBlockInc_
    $define _ORG_SyncBlockInc_

'    GoTo OverORG_SyncBlockInc
'===============================================================================
$if _device = _18F23K20
    $define __DEVICE_ID 263
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F24K20
    $define __DEVICE_ID 261
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F25K20
    $define __DEVICE_ID 259
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F26K20
    $define __DEVICE_ID 257
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F43K20
    $define __DEVICE_ID 262
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F44K20
    $define __DEVICE_ID 260
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F45K20
    $define __DEVICE_ID 258
    $ifndef _erase
        $define _erase 64
    $endif
$endif
$if _device = _18F46K20
    $define __DEVICE_ID 256
    $ifndef _erase
        $define _erase 64
    $endif
$endif

$define ERASE_FLASH_BLOCKSIZE _erase

'===============================================================================
    $ifndef ERASE_FLASH_BLOCKSIZE
    $error "Define the ERASE_FLASH_BLOCKSIZE 64, 128, 256, 512 or 1024 before the Include file"  
    $endif

    $if ERASE_FLASH_BLOCKSIZE > 1024
        $error "ERASE_FLASH_BLOCKSIZE must be 64, 128, 256, 512 or 1024!"
    $endif
    $if ERASE_FLASH_BLOCKSIZE < 64
        $error "ERASE_FLASH_BLOCKSIZE value seems not correct, see the datatsheet!"
    $endif
'-------------------------------------------------------------------------------

    $if ERASE_FLASH_BLOCKSIZE = 1024
'    Symbol ERASE_BLOCK_SHIFT_BITS = 10
    Symbol MASK_ERASE_BLOCK = $FC00
    $endif

    $if ERASE_FLASH_BLOCKSIZE = 512
'    Symbol ERASE_BLOCK_SHIFT_BITS = 9
    Symbol MASK_ERASE_BLOCK = $FE00
    $endif

    $if ERASE_FLASH_BLOCKSIZE = 256
'    Symbol ERASE_BLOCK_SHIFT_BITS = 8
    Symbol MASK_ERASE_BLOCK = $FF00
    $endif

    $if ERASE_FLASH_BLOCKSIZE = 128
'    Symbol ERASE_BLOCK_SHIFT_BITS = 7
    Symbol MASK_ERASE_BLOCK = $FF80
    $endif

    $if ERASE_FLASH_BLOCKSIZE = 64
'    Symbol ERASE_BLOCK_SHIFT_BITS = 6
    Symbol MASK_ERASE_BLOCK = $FFC0
    $endif
'-------------------------------------------------------------------------------
' Syncronize with the ERASE BLOCK    
    $define ORG_EBlock(pAddress) Org (pAddress & MASK_ERASE_BLOCK)    

' Syncronize with the WRITE BLOCK
    $define ORG_WBlock(pAddress) Org (pAddress & -_CWRITE_BLOCK)
    
'===============================================================================    
OverORG_SyncBlockInc:
    $endif    
    
    

